/* fishdebug.c -- fishnet debug/print functions */

#include <stdarg.h>
#include <stdio.h>
#include <time.h>

#include "fish.h"
#include "fishguts.h"

int debuglevel = FISH_DEBUG_ALL;
FILE *debugoutput;

extern void fish_setdebuglevel(const int level) {
  debuglevel = level;
}

extern int fish_getdebuglevel() {
  return debuglevel;

}

extern void fish_setdebugoutput(FILE *f) {
  debugoutput = f;
}

extern void fish_debug(const int level, const char* format, ...) {
  char str[10];
  struct timeval timev;
  va_list ap;
  time_t t;

  if (debuglevel >= level) {
    if(debugoutput == NULL) debugoutput = stderr;
    time(&t);
    strftime(str, 10, "%H:%M:%S", localtime(&t));
    fprintf(debugoutput, str);
    gettimeofday(&timev, NULL);
    fprintf(debugoutput, ".%06ld ", (long)timev.tv_usec);
    
    va_start(ap, format);
    vfprintf(debugoutput, format, ap);
    va_end(ap);

    fflush(debugoutput);
  }
}
    
extern void printmessage(const struct fish_message *pfm) {
  unsigned int i;
  static const char *strmessage[] = {
    "Error",
    "Neighborlist",
    "Join Request"
    };

  printf("Message len: %u, type: %s(%u)\n",
         pfm->length, 
         (pfm->type < FISH_MSG_TYPE_MAX) ? strmessage[pfm->type] 
	                                 : "Invalid type",
         pfm->type);

  switch(pfm->type) {
  case FISH_MSG_TYPE_ERROR:
    printf("Message: %*s\n", 
           pfm->length - MIN_MESSAGE_SIZE,
           pfm->body.errormessage);
    break;
  case FISH_MSG_TYPE_NEIGHBORLIST:
    for(i = 0; 
        i < (pfm->length - MIN_MESSAGE_SIZE)/sizeof(struct neighbor); 
        i++) {
      const struct neighbor *n = &pfm->body.neighbors[i];
      printf("Neighbor %d: fish address %d, udp %s:%u\n",
             i, 
             n->address, 
             inet_ntoa(n->node_ip_address),
             htons(n->node_udp_port));
    }
    break;
  case FISH_MSG_TYPE_JOINREQUEST:
    printf("fish addr %d, udp port %u\n", 
           pfm->body.joinreq.node_address,
           pfm->body.joinreq.node_udp_port);
    break;
  default:
    printf("invalid message.\n");
    break;
  } /* switch */

}


extern void printneighbors(const struct neighbor *ngh) {
  unsigned int i;
  fish_debug(FISH_DEBUG_TOPOLOGY, "fish: Neighbor list \n");
  for(i = 0; 
        i < MAX_NEIGHBORS; 
        i++) {
      if(ngh[i].address >0)
        fish_debug(FISH_DEBUG_TOPOLOGY,
	       "fish:  Neighbor %d: fish address %d, udp %s:%u, loss %f\n",
               i, 
               ngh[i].address, 
               inet_ntoa(ngh[i].node_ip_address),
               htons(ngh[i].node_udp_port),
                   (float) ngh[i].lossrate_x_1000 / (float) 1000);
  }
}
